from wbcore import filters as wb_filters

from wbwriter.models.meta_information import MetaInformationInstance


class MetaInformationInstanceFilter(wb_filters.FilterSet):
    article = wb_filters.ModelChoiceFilter(
        label="Article",
        queryset=MetaInformationInstance.objects.all(),
        endpoint=MetaInformationInstance.get_representation_endpoint(),
        value_key=MetaInformationInstance.get_representation_value_key(),
        label_key=MetaInformationInstance.get_representation_label_key(),
        method="filter_for_article",
    )

    def filter_for_article(self, queryset, name, value):
        if value:
            return MetaInformationInstance.objects.filter(article__id=value)
        return queryset

    class Meta:
        model = MetaInformationInstance
        fields = {
            "article": ["exact"],
        }



from django.db import models


class Style(models.Model):
    """
    A style is a valid CSS construct.
    """

    title = models.CharField(max_length=255, unique=True)
    style = models.TextField(default="")

    def __str__(self):
        return self.title



from wbcore.viewsets import ModelViewSet, RepresentationViewSet

from wbwriter.models import ArticleType
from wbwriter.serializers import (
    ArticleTypeModelSerializer,
    ArticleTypeRepresentationSerializer,
)

from .display import ArticleTypeDisplayConfig


class ArticleTypeModelViewSet(ModelViewSet):
    ENDPOINT = "wbwriter:article-type-list"

    serializer_class = ArticleTypeModelSerializer
    # queryset = ArticleType.objects.all()

    display_config_class = ArticleTypeDisplayConfig

    search_fields = ("label", "slug")
    filter_fields = {"label": ["icontains", "exact"], "slug": ["icontains", "exact"]}
    ordering_fields = ("label",)

    def get_queryset(self):
        request = self.request
        user = request.user
        profile = user.profile

        if user.is_superuser or profile.is_internal:
            return ArticleType.objects.all()
        # TODO: What to do with customers?


class ArticleTypeRepresentationViewSet(RepresentationViewSet):
    serializer_class = ArticleTypeRepresentationSerializer
    queryset = ArticleType.objects.all()

    search_fields = ("label", "slug")
    filter_fields = {"label": ["icontains", "exact"], "slug": ["icontains", "exact"]}
    ordering_fields = ("label",)

    def get_queryset(self):
        request = self.request
        user = request.user
        profile = user.profile

        if user.is_superuser or profile.is_internal:
            return ArticleType.objects.all()

        return ArticleType.objects.none()



from typing import Optional

from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
)
from wbcore.metadata.configs.display.instance_display.utils import repeat_field
from wbcore.metadata.configs.display.view_config import DisplayViewConfig


class MetaInformationDisplayConfig(DisplayViewConfig):
    """Provides getter methods for the list and instance displays."""

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                [repeat_field(2, "article_type")],
                ["name", "key"],
                ["meta_information_type", "boolean_default"],
            ]
        )

    def get_list_display(self) -> Optional[dp.ListDisplay]:
        """Sets up and returns the list display for MetaInformation."""
        return dp.ListDisplay(
            fields=[
                dp.Field(key="id", label="ID"),
                dp.Field(key="article_type", label="Article Type"),
                dp.Field(key="name", label="Name"),
                dp.Field(key="key", label="Key"),
                dp.Field(key="meta_information_type", label="Meta Information Type"),
                dp.Field(key="boolean_default", label="Boolean Default"),
            ],
            formatting=[],
            legends=[],
        )



from wbcore import serializers
from wbcore.contrib.directory.models import Person
from wbcore.contrib.directory.serializers import (
    InternalUserProfileRepresentationSerializer,
)
from wbcore.contrib.icons import WBIcon
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    create_simple_display,
)

from wbwriter.models.article import Article


class ArticleModelButtonConfig(ButtonViewConfig):
    def get_custom_list_instance_buttons(self) -> set:
        return {bt.HyperlinkButton(key="preview", label="Preview PDF", icon=WBIcon.VIEW.icon)}

    def get_custom_instance_buttons(self):
        class AssignAuthorSerializer(serializers.Serializer):
            author = serializers.PrimaryKeyRelatedField(queryset=Person.objects.filter_only_internal())
            _author = InternalUserProfileRepresentationSerializer(source="author")

        custom_instance_buttons = {
            bt.ActionButton(key="generate_pdf", label="Generate PDF", icon=WBIcon.DOCUMENT.icon),
            list(self.get_custom_list_instance_buttons())[0],
            bt.WidgetButton(key="publications", label="Publications", icon=WBIcon.NOTEBOOK.icon),
            bt.ActionButton(key="reroll_peer", label="Assign new peer reviewer", icon=WBIcon.REFRESH.icon),
            bt.ActionButton(key="reroll_qa", label="Assign new QA reviewer", icon=WBIcon.REFRESH.icon),
            bt.ActionButton(
                key="reroll_peer_and_qa", label="Assign new peer and QA reviewers", icon=WBIcon.REFRESH.icon
            ),
            bt.ActionButton(
                key="assign_new_author",
                label="Assign another author",
                icon=WBIcon.PEOPLE.icon,
                serializer=AssignAuthorSerializer,
                instance_display=create_simple_display([["author"]]),
            ),
        }
        # Make sure that the "Edit" button only shows up on Status "draft" and
        # only if the article has no author assigned, and is of Type "DDQ".
        if self.view.kwargs.get("pk", None):
            instance = self.view.get_object()
            if (
                instance.status == Article.Status.DRAFT
                and instance.type
                and instance.type.slug == "ddq"
                and not instance.author
            ):
                custom_instance_buttons.add(bt.ActionButton(key="edit", label="Edit", icon=WBIcon.EDIT.icon))

        return custom_instance_buttons


class PublicationButtonConfig(ButtonViewConfig):
    def get_custom_instance_buttons(self):
        return {bt.HyperlinkButton(key="pdf_file", label="PDF", icon=WBIcon.NOTEBOOK.icon)}

    def get_custom_list_instance_buttons(self):
        return self.get_custom_instance_buttons()



from django.contrib import admin
from reversion_compare.admin import CompareVersionAdmin

from wbwriter.models import (
    Article,
    ArticleType,
    Block,
    BlockParameter,
    DependantArticle,
    InEditorTemplate,
    MetaInformation,
    MetaInformationInstance,
    Publication,
    PublicationParser,
    Style,
    Template,
)


class UsesArticleInline(admin.TabularInline):
    verbose_name = "Uses Article"
    fields = ["dependant_article"]
    model = DependantArticle
    fk_name = "article"
    extra = 0


class UsedArticleInline(admin.TabularInline):
    verbose_name = "Used in Article"
    fields = ["article"]
    model = DependantArticle
    fk_name = "dependant_article"
    extra = 0


@admin.register(MetaInformation)
class MetaInformationModelAdmin(admin.ModelAdmin):
    pass


@admin.register(MetaInformationInstance)
class MetaInformationInstanceModelAdmin(admin.ModelAdmin):
    pass


class MetaInformationInstanceInline(admin.TabularInline):
    model = MetaInformationInstance
    extra = 0
    list_display = ("meta_information", "boolean_value")


@admin.register(ArticleType)
class ArticleTypeModelAdmin(admin.ModelAdmin):
    list_display = ("id", "label", "slug")

    ordering = ["slug"]
    search_fields = ["id", "label", "slug"]

    autocomplete_fields = [
        "peer_reviewers",
        "qa_reviewers",
    ]

    # prepopulated_fields = {"slug": ("label",)}


@admin.register(Article)
class ArticleModelAdmin(CompareVersionAdmin, admin.ModelAdmin):
    list_display = ("id", "title", "slug")

    autocomplete_fields = [
        "author",
        "reviewer",
        "peer_reviewer",
        "qa_reviewer",
    ]

    inlines = [UsesArticleInline, UsedArticleInline, MetaInformationInstanceInline]

    def reversion_register(self, model, **options):
        options = {
            # Foreign keys that will be observed.
            # "follow": ("author", "reviewer", "peer_reviewer", "qa_reviewer"),
            # Fields to ignore when creating versions
            "exclude": ("created", "modified", "published"),
            "ignore_duplicates": True,
        }
        super().reversion_register(model, **options)


@admin.register(Style)
class StyleModelAdmin(admin.ModelAdmin):
    list_display = ("id", "title")


@admin.register(InEditorTemplate)
class InEditorTemplateModelAdmin(admin.ModelAdmin):
    list_display = ("uuid", "title", "is_stand_alone_template")

    ordering = ["uuid", "title", "is_stand_alone_template"]
    search_fields = ["uuid", "title", "description"]


@admin.register(Template)
class TemplateModelAdmin(admin.ModelAdmin):
    list_display = ("id", "title")


class BlockParameterInline(admin.StackedInline):
    model = BlockParameter
    extra = 0


@admin.register(Block)
class BlockModelAdmin(admin.ModelAdmin):
    list_display = ("title",)

    inlines = (BlockParameterInline,)


def rerender_publication(modeladmin, request, queryset):
    for pub in queryset:
        Publication.create_or_update_from_parser_and_object(pub.parser, pub.content_object)


@admin.register(Publication)
class PublicationModelAdmin(CompareVersionAdmin, admin.ModelAdmin):
    list_display = ("id", "title", "modified", "created")
    list_filter = ("created", "modified")

    ordering = ["title", "author", "created", "modified"]
    search_fields = ["title", "author__computed_str"]

    actions = [rerender_publication]


@admin.register(PublicationParser)
class PublicationParserModelAdmin(admin.ModelAdmin):
    list_display = ("title",)
    list_filter = ("title",)

    ordering = ["title", "parser_path"]
    search_fields = ["title", "slug"]



# Generated by Django 4.1.7 on 2023-04-19 05:26

import django.db.models.deletion
import django.utils.timezone
import django_fsm
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("tags", "0001_initial"),
        ("directory", "0001_initial"),
        ("wbcore", "0001_initial_squashed_squashed_0010_preset_appliedpreset"),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Style",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255, unique=True)),
                ("style", models.TextField(default="")),
            ],
        ),
        migrations.CreateModel(
            name="Template",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255, unique=True)),
                ("template", models.TextField(default="")),
                ("styles", models.ManyToManyField(blank=True, related_name="templates", to="wbwriter.style")),
                (
                    "footer_template",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="footer_templates",
                        to="wbwriter.template",
                    ),
                ),
                (
                    "header_template",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="header_templates",
                        to="wbwriter.template",
                    ),
                ),
                ("extra_vertical_margin", models.FloatField(default=10)),
                ("side_margin", models.FloatField(default=2.5)),
            ],
        ),
        migrations.CreateModel(
            name="Block",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=512)),
                ("html", models.TextField(default="")),
                ("key", models.CharField(blank=True, max_length=512, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="BlockParameter",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("order", models.PositiveIntegerField()),
                ("title", models.CharField(max_length=255)),
                ("many", models.BooleanField(default=False)),
                (
                    "block",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, related_name="parameters", to="wbwriter.block"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="InEditorTemplate",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("uuid", models.CharField(max_length=255, unique=True)),
                (
                    "title",
                    models.CharField(
                        help_text="The title should be unique but doesn't need to be.",
                        max_length=255,
                        verbose_name="Title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        default="",
                        help_text="A brief text that describes the use case for this template.",
                        verbose_name="Description",
                    ),
                ),
                (
                    "style",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="The CSS that styles the templates HTML.",
                        null=True,
                        verbose_name="Template CSS",
                    ),
                ),
                (
                    "template",
                    models.TextField(
                        default="", help_text="The HTML code of the template.", verbose_name="Template HTML"
                    ),
                ),
                ("is_stand_alone_template", models.BooleanField(default=True)),
                (
                    "modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The last time this template has been edited.",
                        verbose_name="Last modification date and time",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="PublicationParser",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=1024, unique=True)),
                ("slug", models.CharField(blank=True, max_length=1024)),
                ("parser_path", models.CharField(max_length=1024, unique=True)),
            ],
            options={
                "verbose_name": "PublicationParser",
                "verbose_name_plural": "PublicationParsers",
            },
        ),
        migrations.CreateModel(
            name="ArticleType",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("label", models.CharField(max_length=255, unique=True)),
                ("slug", models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                (
                    "parsers",
                    models.ManyToManyField(
                        blank=True, related_name="publication_parsers", to="wbwriter.publicationparser"
                    ),
                ),
            ],
            options={
                "verbose_name": "Article Type",
                "verbose_name_plural": "Article Types",
            },
        ),
        migrations.CreateModel(
            name="Article",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "title",
                    models.CharField(
                        blank=True,
                        help_text="The title of the article that is going to be used when imported into other articles. Defaults to the name of the article when not set.",
                        max_length=1024,
                        null=True,
                    ),
                ),
                ("slug", models.CharField(blank=True, max_length=1024, null=True)),
                ("content", models.TextField(default="")),
                (
                    "status",
                    django_fsm.FSMField(
                        choices=[
                            ("draft", "Draft"),
                            ("feedback", "Feedback"),
                            ("peer_review", "Peer Review"),
                            ("qa_review", "QA Review"),
                            ("approved", "Approved"),
                        ],
                        default="draft",
                        max_length=50,
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="articles",
                        to="wbwriter.template",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="A unique name to reference this article.", max_length=1024, unique=True
                    ),
                ),
                ("tag_detail_endpoint", models.CharField(blank=True, max_length=255, null=True)),
                ("tag_representation", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "access_level",
                    models.TextField(
                        choices=[("All", "All"), ("Employee", "Employee")], default="Employee", max_length=16
                    ),
                ),
                ("article_structure", models.JSONField(blank=True, null=True)),
                (
                    "created",
                    models.DateField(
                        auto_now_add=True,
                        default=django.utils.timezone.now,
                        help_text="The date on which this article has been created.",
                        verbose_name="Creation Date",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The last time this article has been edited.",
                        verbose_name="Last modification date and time",
                    ),
                ),
                (
                    "published",
                    models.DateTimeField(
                        blank=True,
                        help_text="The date and time on which this article has been/will be published.",
                        null=True,
                        verbose_name="Publication Date and Time",
                    ),
                ),
                (
                    "type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="article",
                        to="wbwriter.articletype",
                    ),
                ),
                (
                    "raw_content",
                    models.JSONField(
                        default=dict,
                        help_text="Maps content IDs to content and configurations to sections.",
                        verbose_name="Raw Content",
                    ),
                ),
                (
                    "in_editor_templates",
                    models.ManyToManyField(
                        blank=True, related_name="templated_articles", to="wbwriter.ineditortemplate"
                    ),
                ),
                ("peer_reviewer_approved", models.BooleanField(default=False)),
                ("qa_reviewer_approved", models.BooleanField(default=False)),
                ("teaser_image", models.ImageField(blank=True, null=True, upload_to="wbwriter/teaser_images")),
            ],
            options={
                "permissions": [("administrate_article", "Can administrate Articles.")],
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
            },
        ),
        migrations.CreateModel(
            name="Publication",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=1024)),
                ("slug", models.CharField(blank=True, max_length=1024)),
                (
                    "created",
                    models.DateField(
                        auto_now_add=True,
                        help_text="The date on which this has been created.",
                        verbose_name="Creation Date",
                    ),
                ),
                ("content", models.TextField(default="")),
                ("content_file", models.FileField(blank=True, null=True, upload_to="")),
                (
                    "parser",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="parsed_publication",
                        to="wbwriter.publicationparser",
                    ),
                ),
                (
                    "content_type",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"),
                ),
                ("object_id", models.PositiveIntegerField()),
                ("tag_detail_endpoint", models.CharField(blank=True, max_length=255, null=True)),
                ("tag_representation", models.CharField(blank=True, max_length=255, null=True)),
                ("teaser_image", models.ImageField(blank=True, null=True, upload_to="wbwriter/publication/assets")),
            ],
            options={
                "verbose_name": "Publication",
                "verbose_name_plural": "Publications",
            },
        ),
        migrations.CreateModel(
            name="MetaInformation",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255)),
                (
                    "meta_information_type",
                    models.CharField(choices=[("BOOLEAN", "Boolean")], default="BOOLEAN", max_length=24),
                ),
                ("boolean_default", models.BooleanField(blank=True, null=True)),
                ("article_type", models.ManyToManyField(related_name="meta_information", to="wbwriter.articletype")),
            ],
            options={
                "verbose_name": "Meta Information",
                "verbose_name_plural": "MetaInformation",
            },
        ),
        migrations.CreateModel(
            name="MetaInformationInstance",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("boolean_value", models.BooleanField(blank=True, null=True)),
                (
                    "article",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="meta_information",
                        to="wbwriter.article",
                    ),
                ),
                (
                    "meta_information",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="instances",
                        to="wbwriter.metainformation",
                    ),
                ),
            ],
            options={
                "verbose_name": "Meta Information Instance",
                "verbose_name_plural": "Meta Information Instances",
            },
        ),
        migrations.AddConstraint(
            model_name="metainformationinstance",
            constraint=models.UniqueConstraint(
                fields=("meta_information", "article"), name="unique_meta_information_article"
            ),
        ),
        migrations.AlterModelOptions(
            name="metainformation",
            options={"verbose_name": "Meta Information", "verbose_name_plural": "Meta Information"},
        ),
        migrations.AddField(
            model_name="metainformation",
            name="key",
            field=models.CharField(default=None, max_length=255, unique=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="metainformation",
            name="name",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AddField(
            model_name="publication",
            name="additional_information",
            field=models.JSONField(blank=True, default=dict, null=True),
        ),
        migrations.AlterModelOptions(
            name="article",
            options={
                "permissions": [
                    ("administrate_article", "Can administrate Articles."),
                    ("publish_article", "Can publish Articles."),
                    ("peer_review_article", "Can peer review Articles."),
                    ("qa_review_article", "Can QA review Articles."),
                ],
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
            },
        ),
        migrations.AlterField(
            model_name="article",
            name="status",
            field=django_fsm.FSMField(
                choices=[
                    ("draft", "Draft"),
                    ("feedback", "Feedback"),
                    ("peer_review", "Peer Review"),
                    ("qa_review", "QA Review"),
                    ("approved", "Approved"),
                    ("published", "Published"),
                ],
                default="draft",
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="article",
            name="status",
            field=django_fsm.FSMField(
                choices=[
                    ("draft", "Draft"),
                    ("feedback", "Feedback"),
                    ("peer_review", "Peer Review"),
                    ("qa_review", "QA Review"),
                    ("author_approval", "Author approval"),
                    ("approved", "Approved"),
                    ("published", "Published"),
                ],
                default="draft",
                max_length=50,
            ),
        ),
        migrations.RemoveField(
            model_name="article",
            name="template",
        ),
        migrations.AddField(
            model_name="publication",
            name="target",
            field=models.CharField(default="website", max_length=256),
            preserve_default=False,
        ),
        migrations.RemoveField(
            model_name="article",
            name="qa_reviewer_approved",
        ),
        migrations.AlterField(
            model_name="article",
            name="type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT, related_name="article", to="wbwriter.articletype"
            ),
        ),
        migrations.AddField(
            model_name="article",
            name="template",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="articles",
                to="wbwriter.template",
            ),
        ),
        migrations.RemoveField(
            model_name="article",
            name="access_level",
        ),
        migrations.AddField(
            model_name="article",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="Signifies whether this article can be seen by all customers on the homepage."
            ),
        ),
        migrations.RemoveField(
            model_name="article",
            name="in_editor_templates",
        ),
        migrations.RemoveField(
            model_name="article",
            name="content",
        ),
        migrations.RenameField(
            model_name="article",
            old_name="raw_content",
            new_name="content",
        ),
        migrations.AlterField(
            model_name="article",
            name="content",
            field=models.JSONField(default=dict, verbose_name="Content"),
        ),
        migrations.AddField(
            model_name="publication",
            name="modified",
            field=models.DateTimeField(
                auto_now=True,
                help_text="The date and time on which this has been modified last.",
                verbose_name="Last Modification Datetime",
            ),
        ),
        migrations.RemoveField(
            model_name="article",
            name="published",
        ),
        migrations.AddField(
            model_name="publication",
            name="description",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="article",
            name="teaser_image",
            field=models.ImageField(blank=True, null=True, upload_to="writer/article/teasers"),
        ),
        migrations.AlterField(
            model_name="publication",
            name="content_file",
            field=models.FileField(
                blank=True, max_length=256, null=True, upload_to="writer/publication/content_files"
            ),
        ),
        migrations.AlterField(
            model_name="publication",
            name="teaser_image",
            field=models.ImageField(blank=True, null=True, upload_to="writer/publication/teasers"),
        ),
        migrations.AddField(
            model_name="publication",
            name="thumbnail_image",
            field=models.ImageField(blank=True, null=True, upload_to="writer/publication/thumbnails"),
        ),
        migrations.AlterField(
            model_name="article",
            name="tag_representation",
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
        migrations.AlterField(
            model_name="publication",
            name="tag_representation",
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
        migrations.AlterModelOptions(
            name="article",
            options={
                "permissions": [
                    ("administrate_article", "Can administrate Articles."),
                    ("assign_article_reviewers", "Can assign Articles' peer and QA reviewers."),
                    ("publish_article", "Can publish Articles."),
                    ("peer_review_article", "Can peer review Articles."),
                    ("qa_review_article", "Can QA review Articles."),
                ],
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
            },
        ),
        migrations.AddField(
            model_name="article",
            name="tags",
            field=models.ManyToManyField(blank=True, related_name="%(app_label)s_%(class)s_items", to="tags.tag"),
        ),
        migrations.AddField(
            model_name="publication",
            name="tags",
            field=models.ManyToManyField(blank=True, related_name="%(app_label)s_%(class)s_items", to="tags.tag"),
        ),
        migrations.AddField(
            model_name="article",
            name="author",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="author_articles",
                to="directory.person",
            ),
        ),
        migrations.AddField(
            model_name="article",
            name="feedback_contact",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="feedback_contact_articles",
                to="directory.person",
            ),
        ),
        migrations.AddField(
            model_name="article",
            name="peer_reviewer",
            field=models.ForeignKey(
                blank=True,
                help_text="The peer reviewer who reviewed this article.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="peer_review_articles",
                to="directory.person",
            ),
        ),
        migrations.AddField(
            model_name="article",
            name="qa_reviewer",
            field=models.ForeignKey(
                blank=True,
                help_text="The quality assurance (QA) reviewer who reviewed this article.",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="qa_review_articles",
                to="directory.person",
            ),
        ),
        migrations.AddField(
            model_name="article",
            name="reviewer",
            field=models.ForeignKey(
                blank=True,
                help_text="The contact that is currently working on feedback.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="review_articles",
                to="directory.person",
            ),
        ),
        migrations.AddField(
            model_name="articletype",
            name="peer_reviewers",
            field=models.ManyToManyField(related_name="article_type_peer_reviewers", to="directory.person"),
        ),
        migrations.AddField(
            model_name="articletype",
            name="qa_reviewers",
            field=models.ManyToManyField(related_name="article_type_qa_reviewers", to="directory.person"),
        ),
        migrations.AddField(
            model_name="publication",
            name="author",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="publication",
                to="directory.person",
            ),
        ),
        migrations.AlterField(
            model_name="article",
            name="created",
            field=models.DateField(
                auto_now_add=True,
                help_text="The date on which this article has been created.",
                verbose_name="Creation Date",
            ),
        ),
    ]



from unittest.mock import patch

import pytest
from django.forms.models import model_to_dict
from django_fsm import has_transition_perm
from wbcore.contrib.authentication.factories import UserFactory
from wbcore.contrib.authentication.models import Permission, User
from wbwriter.factories import (
    ArticleFactory,
    ArticleTypeFactory,
    DependantArticleFactory,
    MetaInformationInstanceFactory,
)
from wbwriter.models import ArticleType, Publication
from wbwriter.models.article import (
    Article,
    DependantArticle,
    can_access_article,
    can_administrate_article,
    can_edit_article_content,
    can_edit_article_meta_data,
    can_edit_article_type,
    can_request_peer_review,
    generate_publications,
)
from wbwriter.models.meta_information import MetaInformationInstance


@pytest.mark.django_db
class TestArticleTransitions:
    @pytest.mark.parametrize("author", [True, False])
    def test_requestfeedback(self, author):
        user = UserFactory(is_superuser=False)
        if author:
            article = ArticleFactory(status=Article.Status.DRAFT, author=user.profile)
            assert has_transition_perm(article.requestfeedback, user) is True
        else:
            article = ArticleFactory(status=Article.Status.DRAFT)
            assert has_transition_perm(article.requestfeedback, user) is False

    @pytest.mark.parametrize("reviewer", [True, False])
    def test_submitfeedback(self, reviewer):
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        article = ArticleFactory(
            status=Article.Status.FEEDBACK,
            author=author_user.profile,
            reviewer=reviewer_user.profile if reviewer else UserFactory().profile,
        )
        if reviewer:
            assert has_transition_perm(article.submitfeedback, reviewer_user) is True
            assert has_transition_perm(article.submitfeedback, author_user) is False
        else:
            article = ArticleFactory(status=Article.Status.FEEDBACK)
            assert has_transition_perm(article.submitfeedback, reviewer_user) is False

    @pytest.mark.parametrize("author", [True, False])
    def test_requestpeerreview(self, author):
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        article_review_approved = ArticleFactory(
            status=Article.Status.DRAFT,
            author=author_user.profile if author else UserFactory().profile,
            reviewer=reviewer_user.profile,
            peer_reviewer_approved=True,
        )
        article_not_review_approved = ArticleFactory(
            status=Article.Status.DRAFT,
            author=author_user.profile if author else UserFactory().profile,
            reviewer=reviewer_user.profile,
            peer_reviewer_approved=False,
        )
        if author:
            assert has_transition_perm(article_review_approved.requestpeerreview, author_user) is False
            assert has_transition_perm(article_review_approved.requestpeerreview, reviewer_user) is False
            assert has_transition_perm(article_not_review_approved.requestpeerreview, author_user) is True
        else:
            assert has_transition_perm(article_review_approved.requestpeerreview, author_user) is False
            assert has_transition_perm(article_not_review_approved.requestpeerreview, author_user) is False

    @pytest.mark.parametrize("author", [True, False])
    def test_requestqareview(self, author):
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        article_review_approved = ArticleFactory(
            status=Article.Status.DRAFT,
            author=author_user.profile if author else UserFactory().profile,
            reviewer=reviewer_user.profile,
            peer_reviewer_approved=True,
        )
        article_not_review_approved = ArticleFactory(
            status=Article.Status.DRAFT,
            author=author_user.profile if author else UserFactory().profile,
            reviewer=reviewer_user.profile,
            peer_reviewer_approved=False,
        )
        if author:
            assert has_transition_perm(article_review_approved.requestqareview, author_user) is True
            assert has_transition_perm(article_not_review_approved.requestqareview, author_user) is False
            assert has_transition_perm(article_review_approved.requestqareview, reviewer_user) is False
        else:
            assert has_transition_perm(article_review_approved.requestqareview, author_user) is False
            assert has_transition_perm(article_not_review_approved.requestqareview, author_user) is False

    def test_peerapprove(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user_a = UserFactory(is_superuser=False)
        peer_reviewer_user_b = UserFactory(is_superuser=False)
        article_type_a = ArticleTypeFactory(peer_reviewers=[peer_reviewer_user_a.profile])
        ArticleTypeFactory(peer_reviewers=[peer_reviewer_user_b.profile])

        article = ArticleFactory(
            status=Article.Status.PEER_REVIEW,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user_a.profile,
            type=article_type_a,
        )

        assert has_transition_perm(article.peerapprove, author_user) is False
        assert has_transition_perm(article.peerapprove, reviewer_user) is False
        assert has_transition_perm(article.peerapprove, peer_reviewer_user_b) is False
        assert has_transition_perm(article.peerapprove, peer_reviewer_user_a) is True
        assert has_transition_perm(article.peerapprove, super_user) is True

    def test_peerreject(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user_a = UserFactory(is_superuser=False)
        peer_reviewer_user_b = UserFactory(is_superuser=False)
        ArticleTypeFactory(peer_reviewers=[peer_reviewer_user_b.profile])
        article_type_a = ArticleTypeFactory(peer_reviewers=[peer_reviewer_user_a.profile])

        article = ArticleFactory(
            status=Article.Status.PEER_REVIEW,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user_a.profile,
            type=article_type_a,
        )

        assert has_transition_perm(article.peerapprove, author_user) is False
        assert has_transition_perm(article.peerapprove, reviewer_user) is False
        assert has_transition_perm(article.peerapprove, peer_reviewer_user_b) is False
        assert has_transition_perm(article.peerapprove, peer_reviewer_user_a) is True
        assert has_transition_perm(article.peerapprove, super_user) is True

    def test_qa_reject(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user_a = UserFactory(is_superuser=False)
        qa_reviewer_user_b = UserFactory(is_superuser=False)
        ArticleTypeFactory(qa_reviewers=[qa_reviewer_user_b.profile])

        article_type_a = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user_a.profile],
        )

        article = ArticleFactory(
            status=Article.Status.QA_REVIEW,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user_a.profile,
            type=article_type_a,
        )

        assert has_transition_perm(article.qareject, author_user) is False
        assert has_transition_perm(article.qareject, reviewer_user) is False
        assert has_transition_perm(article.qareject, peer_reviewer_user) is False
        assert has_transition_perm(article.qareject, qa_reviewer_user_b) is False
        assert has_transition_perm(article.qareject, super_user) is True
        assert has_transition_perm(article.qareject, qa_reviewer_user_a) is True

    def test_qa_approve(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user_a = UserFactory(is_superuser=False)
        qa_reviewer_user_b = UserFactory(is_superuser=False)
        ArticleTypeFactory(qa_reviewers=[qa_reviewer_user_b.profile])

        article_type_a = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user_a.profile],
        )

        article = ArticleFactory(
            status=Article.Status.QA_REVIEW,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user_a.profile,
            type=article_type_a,
        )

        assert has_transition_perm(article.qaapprove, author_user) is False
        assert has_transition_perm(article.qaapprove, reviewer_user) is False
        assert has_transition_perm(article.qaapprove, peer_reviewer_user) is False
        assert has_transition_perm(article.qaapprove, qa_reviewer_user_b) is False
        assert has_transition_perm(article.qaapprove, super_user) is True
        assert has_transition_perm(article.qaapprove, qa_reviewer_user_a) is True

    def test_authorreject(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user = UserFactory(is_superuser=False)
        article_type_a = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user.profile],
        )
        article = ArticleFactory(
            status=Article.Status.AUTHOR_APPROVAL,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user.profile,
            type=article_type_a,
        )

        assert has_transition_perm(article.authorreject, reviewer_user) is False
        assert has_transition_perm(article.authorreject, peer_reviewer_user) is False
        assert has_transition_perm(article.authorreject, qa_reviewer_user) is False
        assert has_transition_perm(article.authorreject, super_user) is True
        assert has_transition_perm(article.authorreject, author_user) is True

    def test_authorapprove(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user = UserFactory(is_superuser=False)
        article_type_a = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user.profile],
        )
        article = ArticleFactory(
            status=Article.Status.AUTHOR_APPROVAL,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user.profile,
            type=article_type_a,
        )

        assert has_transition_perm(article.authorapprove, reviewer_user) is False
        assert has_transition_perm(article.authorapprove, peer_reviewer_user) is False
        assert has_transition_perm(article.authorapprove, qa_reviewer_user) is False
        assert has_transition_perm(article.authorapprove, super_user) is True
        assert has_transition_perm(article.authorapprove, author_user) is True

    def test_publish(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user = UserFactory(is_superuser=False)
        can_publish_user: User = UserFactory(is_superuser=False)
        can_publish_user.user_permissions.add(Permission.objects.get(codename="administrate_article"))

        article_type = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user.profile],
            label="one-off-article",
        )
        article = ArticleFactory(
            status=Article.Status.APPROVED,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user.profile,
            type=article_type,
        )
        assert has_transition_perm(article.publish, reviewer_user) is False
        assert has_transition_perm(article.publish, peer_reviewer_user) is False
        assert has_transition_perm(article.publish, qa_reviewer_user) is False
        assert has_transition_perm(article.publish, author_user) is False
        assert has_transition_perm(article.publish, super_user) is True
        assert has_transition_perm(article.publish, can_publish_user) is True

    def test_unpublish(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user = UserFactory(is_superuser=False)
        can_publish_user: User = UserFactory(is_superuser=False)
        can_publish_user.user_permissions.add(Permission.objects.get(codename="administrate_article"))

        article_type = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user.profile],
            label="one-off-article",
        )
        article = ArticleFactory(
            status=Article.Status.PUBLISHED,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user.profile,
            type=article_type,
        )
        assert has_transition_perm(article.unpublish, reviewer_user) is False
        assert has_transition_perm(article.unpublish, peer_reviewer_user) is False
        assert has_transition_perm(article.unpublish, qa_reviewer_user) is False
        assert has_transition_perm(article.unpublish, author_user) is False
        assert has_transition_perm(article.unpublish, super_user) is True
        assert has_transition_perm(article.unpublish, can_publish_user) is True

    def test_authorrevise(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user = UserFactory(is_superuser=False)
        administrate_user: User = UserFactory(is_superuser=False)
        administrate_user.user_permissions.add(Permission.objects.get(codename="administrate_article"))

        article_type = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user.profile],
            label="one-off-article",
        )
        article = ArticleFactory(
            status=Article.Status.APPROVED,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user.profile,
            type=article_type,
        )
        assert has_transition_perm(article.authorrevise, reviewer_user) is False
        assert has_transition_perm(article.authorrevise, peer_reviewer_user) is False
        assert has_transition_perm(article.authorrevise, qa_reviewer_user) is False
        assert has_transition_perm(article.authorrevise, author_user) is True
        assert has_transition_perm(article.authorrevise, super_user) is True
        assert has_transition_perm(article.authorrevise, administrate_user) is True

    def test_qarevise(self):
        super_user = UserFactory(is_superuser=True)
        author_user = UserFactory(is_superuser=False)
        reviewer_user = UserFactory(is_superuser=False)
        peer_reviewer_user = UserFactory(is_superuser=False)
        qa_reviewer_user_a = UserFactory(is_superuser=False)
        qa_reviewer_user_b = UserFactory(is_superuser=False)
        administrate_user: User = UserFactory(is_superuser=False)
        administrate_user.user_permissions.add(Permission.objects.get(codename="administrate_article"))

        article_type = ArticleTypeFactory(
            peer_reviewers=[peer_reviewer_user.profile],
            qa_reviewers=[qa_reviewer_user_a.profile, qa_reviewer_user_b.profile],
            label="one-off-article",
        )
        article = ArticleFactory(
            status=Article.Status.APPROVED,
            author=author_user.profile,
            reviewer=reviewer_user.profile,
            peer_reviewer=peer_reviewer_user.profile,
            qa_reviewer=qa_reviewer_user_a.profile,
            type=article_type,
        )
        assert has_transition_perm(article.qarevise, reviewer_user) is False
        assert has_transition_perm(article.qarevise, peer_reviewer_user) is False
        assert has_transition_perm(article.qarevise, qa_reviewer_user_a) is True
        assert has_transition_perm(article.qarevise, qa_reviewer_user_b) is True
        assert has_transition_perm(article.qarevise, author_user) is False
        assert has_transition_perm(article.qarevise, super_user) is True
        assert has_transition_perm(article.qarevise, administrate_user) is True


@pytest.mark.django_db
class TestArticlePermissions:
    def test_can_administrate_article(self):
        super_user: User = UserFactory(is_superuser=True)
        article_admin_user: User = UserFactory(is_superuser=False)
        article_admin_user.user_permissions.add(Permission.objects.get(codename="administrate_article"))
        unalligned_user: User = UserFactory(is_superuser=False)

        article = ArticleFactory()
        assert can_administrate_article(article, super_user)
        assert can_administrate_article(article, article_admin_user)
        assert not can_administrate_article(article, unalligned_user)

    def test_can_publish_articles(self):
        super_user: User = UserFactory(is_superuser=True)
        article_admin_user: User = UserFactory(is_superuser=False)
        article_admin_user.user_permissions.add(Permission.objects.get(codename="administrate_article"))
        unalligned_user: User = UserFactory(is_superuser=False)

        article: Article = ArticleFactory()
        assert can_administrate_article(article, super_user)
        assert can_administrate_article(article, article_admin_user)
        assert not can_administrate_article(article, unalligned_user)

    @patch.object(Publication, "create_or_update_from_parser_and_object")
    def test_can_be_published(self, mock_fct):
        article_a: Article = ArticleFactory(type=ArticleTypeFactory(label="One Off Article"))

        assert article_a.can_be_published() is False
        generate_publications(article_a.id)
        assert mock_fct.call_count == 0

        article_a.status = Article.Status.APPROVED
        article_a.save()
        assert article_a.can_be_published()
        generate_publications(article_a.id)
        assert mock_fct.call_count == 1

        article_b: Article = ArticleFactory(
            status=Article.Status.APPROVED, type=ArticleTypeFactory(label="Deep Dive Article")
        )
        article_c: Article = ArticleFactory(
            status=Article.Status.APPROVED, type=ArticleTypeFactory(label="Mid Year Review")
        )
        article_d: Article = ArticleFactory(
            status=Article.Status.APPROVED, type=ArticleTypeFactory(label="Unalligned Article")
        )
        assert article_a.can_be_published()
        assert article_b.can_be_published()
        assert article_c.can_be_published()
        generate_publications(article_a.id)
        generate_publications(article_b.id)
        generate_publications(article_c.id)
        assert mock_fct.call_count == 4
        assert not article_d.can_be_published()
        generate_publications(article_d.id)
        assert mock_fct.call_count == 4

    def test_can_access_article(self):
        super_user: User = UserFactory(is_superuser=True)
        reviewer_user: User = UserFactory(is_superuser=False)
        is_qa_reviewer_user: User = UserFactory(is_superuser=False)
        is_peer_reviewer_user: User = UserFactory(is_superuser=False)
        user_can_administrate_article: User = UserFactory(is_superuser=False)
        unalligned_user: User = UserFactory(is_superuser=False)

        user_can_administrate_article.user_permissions.add(Permission.objects.get(codename="administrate_article"))
        article_type: ArticleType = ArticleTypeFactory(
            peer_reviewers=[is_peer_reviewer_user.profile], qa_reviewers=[is_qa_reviewer_user.profile]
        )
        article: Article = ArticleFactory(type=article_type, reviewer=reviewer_user.profile)

        assert can_access_article(article, super_user)
        # assert can_access_article(article, employee_user)
        assert can_access_article(article, is_qa_reviewer_user)
        assert can_access_article(article, is_peer_reviewer_user)
        assert can_access_article(article, user_can_administrate_article)
        assert not can_access_article(article, unalligned_user)

    def test_can_edit_article_author(self):
        self.test_can_administrate_article()

    def test_can_edit_article_content(self):
        user = UserFactory(is_superuser=False)
        user_unalligned = UserFactory(is_superuser=False)
        user_su = UserFactory(is_superuser=True)

        article_type = ArticleTypeFactory(peer_reviewers=[user.profile], qa_reviewers=[user.profile])

        article_draft: Article = ArticleFactory(status=Article.Status.DRAFT, author=user.profile)
        article_feedback: Article = ArticleFactory(status=Article.Status.FEEDBACK, reviewer=user.profile)
        article_peer_review: Article = ArticleFactory(
            status=Article.Status.PEER_REVIEW, peer_reviewer=user.profile, type=article_type
        )
        article_qa_review: Article = ArticleFactory(
            status=Article.Status.QA_REVIEW, qa_reviewer=user.profile, type=article_type
        )
        article_approved: Article = ArticleFactory(status=Article.Status.APPROVED)

        assert can_edit_article_content(article_draft, user)
        assert not can_edit_article_content(article_draft, user_unalligned)
        assert can_edit_article_content(article_feedback, user)
        assert not can_edit_article_content(article_feedback, user_unalligned)
        assert can_edit_article_content(article_peer_review, user)
        assert not can_edit_article_content(article_peer_review, user_unalligned)
        assert can_edit_article_content(article_qa_review, user)
        assert not can_edit_article_content(article_qa_review, user_unalligned)
        assert can_edit_article_content(article_approved, user_su)
        assert not can_edit_article_content(article_approved, user_unalligned)

    def test_can_edit_article_meta_data(self):
        user = UserFactory(is_superuser=False)
        user_unalligned = UserFactory(is_superuser=False)

        article_type = ArticleTypeFactory(peer_reviewers=[user.profile], qa_reviewers=[user.profile])

        article_draft: Article = ArticleFactory(status=Article.Status.DRAFT, author=user.profile)
        article_feedback: Article = ArticleFactory(status=Article.Status.FEEDBACK, reviewer=user.profile)
        article_peer_review: Article = ArticleFactory(
            status=Article.Status.PEER_REVIEW, peer_reviewer=user.profile, type=article_type
        )
        article_qa_review: Article = ArticleFactory(
            status=Article.Status.QA_REVIEW, qa_reviewer=user.profile, type=article_type
        )

        assert can_edit_article_meta_data(article_draft, user=user)
        assert not can_edit_article_meta_data(article_draft, user=user_unalligned)
        assert can_edit_article_meta_data(article_feedback, user=user)
        assert not can_edit_article_meta_data(article_feedback, user=user_unalligned)
        assert can_edit_article_meta_data(article_peer_review, user=user)
        assert not can_edit_article_meta_data(article_peer_review, user=user_unalligned)
        assert can_edit_article_meta_data(article_qa_review, user=user)
        assert not can_edit_article_meta_data(article_qa_review, user=user_unalligned)

    def test_can_edit_article_type(self):
        user = UserFactory(is_superuser=False)
        user_unalligned = UserFactory(is_superuser=False)
        article_draft: Article = ArticleFactory(status=Article.Status.DRAFT, author=user.profile)
        article_feedback: Article = ArticleFactory(status=Article.Status.FEEDBACK, author=user.profile)

        assert can_edit_article_type(article_draft, user)
        assert not can_edit_article_type(article_feedback, user)
        assert not can_edit_article_type(article_draft, user_unalligned)

    @pytest.mark.parametrize("peer_reviewer_approved", [True, False])
    def test_can_request_peer_review(self, peer_reviewer_approved):
        article = ArticleFactory(peer_reviewer_approved=peer_reviewer_approved)
        if peer_reviewer_approved:
            assert not can_request_peer_review(article)
        else:
            assert can_request_peer_review(article)


@pytest.mark.django_db
class TestArticleCloning:
    def test_clone(self, user):
        article = ArticleFactory()
        article = Article.objects.get(
            id=article.id
        )  # To get updated tag_detail_endpoint and tag_representation from db
        meta_information_instance = MetaInformationInstanceFactory(article=article)
        dependender_relationship = DependantArticleFactory(article=article)
        dependendee_relationship = DependantArticleFactory(dependant_article=article)
        assert Article.objects.count() == 3
        assert DependantArticle.objects.count() == 2
        assert MetaInformationInstance.objects.count() == 1

        cloned_article = article.clone(user=user)

        # Article was correctly cloned
        article_dict = model_to_dict(article)
        del article_dict["id"]
        del article_dict["teaser_image"]
        article_clone_dict = model_to_dict(cloned_article)
        assert article_clone_dict
        del article_clone_dict["id"]
        del article_clone_dict["teaser_image"]
        article_dict["name"] += "_clone"
        article_dict["slug"] += "-clone"
        article_dict["slug_en_us"] += "-clone"
        article_dict["slug_i18n"] += "-clone"
        article_dict["title"] += " (Clone)"
        article_dict["title_i18n"] += " (Clone)"
        article_dict["title_en_us"] += " (Clone)"
        article_dict["author"] = user.profile.id
        assert article_dict == article_clone_dict
        assert Article.objects.count() == 4

        # Related dependant articles were correctly cloned
        dependender_relationship_dict = model_to_dict(dependender_relationship)
        del dependender_relationship_dict["id"]
        dependender_relationship_clone_dict = model_to_dict(DependantArticle.objects.get(id=3))
        assert dependender_relationship_clone_dict
        del dependender_relationship_clone_dict["id"]
        dependender_relationship_clone_dict["article"] = article.id
        assert dependender_relationship_dict == dependender_relationship_clone_dict

        dependendee_relationship_dict = model_to_dict(dependendee_relationship)
        del dependendee_relationship_dict["id"]
        dependendee_relationship_clone_dict = model_to_dict(DependantArticle.objects.get(id=4))
        assert dependendee_relationship_clone_dict
        del dependendee_relationship_clone_dict["id"]
        dependendee_relationship_clone_dict["dependant_article"] = article.id
        assert dependendee_relationship_dict == dependendee_relationship_clone_dict

        assert DependantArticle.objects.count() == 4

        # Related meta information instances were correctly cloned
        meta_information_instance_dict = model_to_dict(meta_information_instance)
        del meta_information_instance_dict["id"]
        meta_information_instance_clone_dict = model_to_dict(MetaInformationInstance.objects.last())
        assert meta_information_instance_clone_dict
        del meta_information_instance_clone_dict["id"]
        meta_information_instance_clone_dict["article"] = article.id
        assert meta_information_instance_dict == meta_information_instance_clone_dict
        assert MetaInformationInstance.objects.count() == 2



from dynamic_preferences.preferences import Section
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import IntegerPreference

general = Section("wbwriter")


@global_preferences_registry.register
class ReviewerRollDaysRangePreference(IntegerPreference):
    section = general
    name = "reviewer_roll_days_range"
    default = 31

    verbose_name = "Reviewer roll days range"
    help_text = "The number of days that the reviewer-picking-algorithm looks into the past to count the reviews a member of the reviewer groups has been given."



from rest_framework.reverse import reverse
from wbcore.metadata.configs.endpoints import EndpointViewConfig


class DependantArticleEndpointViewConfig(EndpointViewConfig):
    def _get_instance_endpoint(self, **kwargs):
        return "{{dependant_article_url}}" if "article_id" in self.view.kwargs else "{{article_url}}"


class ReviewerArticleModelEndpointConfig(EndpointViewConfig):
    def get_list_endpoint(self, **kwargs):
        return reverse("wbwriter:review-article-list", args=[], request=self.request)



from .article import (
    AbstractPublicationFactory,
    ArticleFactory,
    ArticleTypeFactory,
    BlockFactory,
    BlockParameterFactory,
    InEditorTemplateFactory,
    PublicationFactory,
    PublicationParserFactory,
    TemplateFactory,
    DependantArticleFactory,
)
from .meta_information import MetaInformationFactory, MetaInformationInstanceFactory



from django.contrib.contenttypes.models import ContentType
from django.db.models import Case, Q, QuerySet, Value, When
from django.shortcuts import get_object_or_404
from django.utils.functional import cached_property
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from reversion.views import RevisionMixin
from wbcore.contrib.authentication.authentication import JWTCookieAuthentication
from wbcore.contrib.directory.models import Person
from wbcore.contrib.i18n.viewsets import ModelTranslateMixin
from wbcore.contrib.icons import WBIcon
from wbcore.utils.views import CloneMixin
from wbcore.viewsets import ModelViewSet, RepresentationViewSet

from wbwriter.filters.article import ArticleFilter
from wbwriter.models import Article, can_access_article
from wbwriter.models.article import (
    DependantArticle,
    can_administrate_article,
    can_edit_article_author,
    can_edit_article_content,
    can_edit_article_meta_data,
    can_edit_article_type,
    generate_pdf_as_task,
)
from wbwriter.serializers import (
    ArticleFullModelSerializer,
    ArticleRepresentionSerializer,
    DependantArticleModelSerializer,
)

from .buttons import ArticleModelButtonConfig
from .display import ArticleDisplayConfig, DependantArticleDisplayConfig
from .endpoints import (
    DependantArticleEndpointViewConfig,
    ReviewerArticleModelEndpointConfig,
)
from .titles import ReviewerArticleTitleConfig


class ArticleRepresentionViewSet(RepresentationViewSet):
    queryset = Article.objects.all()
    serializer_class = ArticleRepresentionSerializer


class DependantArticleModelViewSet(ModelViewSet):
    queryset = DependantArticle.objects.all()
    serializer_class = DependantArticleModelSerializer
    display_config_class = DependantArticleDisplayConfig
    endpoint_config_class = DependantArticleEndpointViewConfig

    def get_queryset(self) -> QuerySet[DependantArticle]:
        queryset = super().get_queryset()
        if article_id := self.kwargs.get("article_id", None):
            queryset = queryset.filter(article_id=article_id)

        if used_article_id := self.kwargs.get("used_article_id", None):
            queryset = queryset.filter(dependant_article_id=used_article_id)

        return queryset.select_related("article", "dependant_article")


class ArticleInstancePermissionMixin:
    @cached_property
    def profile(self):
        user = self.request.user
        return user.profile

    @cached_property
    def instance(self) -> Article | None:
        if "pk" in self.kwargs:
            return self.get_object()

    @cached_property
    def can_edit_article_author(self) -> bool:
        if self.instance:
            return can_edit_article_author(self.instance, self.request.user)
        return False

    @cached_property
    def can_edit_article_meta_data(self) -> bool:
        if self.instance:
            return can_edit_article_meta_data(self.instance, self.request.user)
        return False

    @cached_property
    def can_edit_article_type(self) -> bool:
        return can_edit_article_type(self.instance, self.request.user)

    @cached_property
    def can_edit_article_content(self) -> bool:
        if self.instance:
            return can_edit_article_content(self.instance, self.request.user)
        return False

    @cached_property
    def can_administrate_article(self) -> bool:
        if self.instance:
            return can_administrate_article(self.instance, self.request.user)
        return False


class ArticleModelViewSet(
    ModelTranslateMixin, ArticleInstancePermissionMixin, CloneMixin, RevisionMixin, ModelViewSet
):
    # LIST_DOCUMENTATION = "wbwriter/viewsets/documentation/article.md"

    serializer_class = ArticleFullModelSerializer
    queryset = Article.objects.annotate(
        is_private_icon=Case(When(is_private=False, then=Value(WBIcon.VIEW.icon)), default=Value(WBIcon.IGNORE.icon))
    )

    button_config_class = ArticleModelButtonConfig
    display_config_class = ArticleDisplayConfig
    filterset_class = ArticleFilter

    search_fields = (
        "tags__title",
        "type__label",
        "type__slug",
        "content",
        "title",
        "name",
    )
    ordering = ("-modified",)
    ordering_fields = (
        "title",
        "status",
        "type",
        "created",
        "modified",
        "id",
        "author",
        "reviewer",
        "peer_reviewer",
        "qa_reviewer",
    )

    @cached_property
    def content_type(self):
        return ContentType.objects.get(app_label="wbwriter", model="article")

    def get_queryset(self):
        qs = super().get_queryset()
        if not self.request.user.has_perm("wbwriter.administrate_article") and not self.profile.is_internal:
            qs = qs.filter(is_private=False)
        return qs.select_related(
            "type", "feedback_contact", "reviewer", "peer_reviewer", "qa_reviewer", "author"
        ).prefetch_related("tags")

    @action(detail=True, methods=["POST"], authentication_classes=[JWTCookieAuthentication])
    def generate_pdf(self, request, pk):
        """
        NOTE: JWTCookieAuthentication only works, if the frontend is served under the same URL as the backend due to
              the cookie.
        """
        article = get_object_or_404(Article, pk=pk)

        if not can_access_article(article, request.user):
            return Response({}, status=status.HTTP_403_FORBIDDEN)

        generate_pdf_as_task.delay(pk, user_id=request.user.id)
        return Response(
            {"__notification": {"title": "PDF is going to be created"}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["POST"], authentication_classes=[JWTCookieAuthentication])
    def edit(self, request, pk):
        """Sets the current user to be the author of the specified article."""
        article = get_object_or_404(Article, pk=pk)
        article.author = request.user.profile
        article.save()
        return Response(
            {"__notification": {"title": f'You are now author of "{article.title}".'}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["POST"], authentication_classes=[JWTCookieAuthentication])
    def reroll_peer(self, request, pk):
        """Rolls for a new peer reviewer."""
        article = get_object_or_404(Article, pk=pk)
        article.reroll_peer()
        article.save()
        return Response(
            {"__notification": {"title": f'The new peer reviewer is "{article.peer_reviewer}".'}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["POST"], authentication_classes=[JWTCookieAuthentication])
    def reroll_qa(self, request, pk):
        """Rolls for a new QA reviewer."""
        article = get_object_or_404(Article, pk=pk)
        article.reroll_qa()
        article.save()
        return Response(
            {"__notification": {"title": f'The new QA reviewer is "{article.qa_reviewer}".'}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["POST"], authentication_classes=[JWTCookieAuthentication])
    def reroll_peer_and_qa(self, request, pk):
        """Rolls for a new peer and a new QA reviewer."""
        article = get_object_or_404(Article, pk=pk)
        article.reroll_peer_and_qa()
        article.save()
        return Response(
            {"__notification": {"title": f'You are now author of "{article.title}".'}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["POST"], authentication_classes=[JWTCookieAuthentication])
    def assign_new_author(self, request, pk):
        """Assigns a new author."""
        article = get_object_or_404(Article, pk=pk)
        author_id = request.POST.get("author")
        author = get_object_or_404(Person, pk=author_id)
        article.author = author
        article.save()
        return Response(
            {"__notification": {"title": f"The new author is {author}."}},
            status=status.HTTP_200_OK,
        )


class ReviewerArticleModelViewSet(ArticleModelViewSet):
    # LIST_DOCUMENTATION = "wbwriter/viewsets/documentation/article.md"

    # serializer_class = ArticleModelSerializer
    queryset = Article.objects.all()
    title_config_class = ReviewerArticleTitleConfig

    button_config_class = ArticleModelButtonConfig
    display_config_class = ArticleDisplayConfig
    endpoint_config_class = ReviewerArticleModelEndpointConfig

    search_fields = (
        "tags__title",
        "type__label",
        "type__slug",
        "content",
        "title",
        "name",
    )
    filter_fields = {
        "title": ["icontains"],
        "status": ["exact"],
        "tags": ["exact"],
        "type": ["exact", "icontains"],
        "author": ["exact"],
        "reviewer": ["exact"],
        "peer_reviewer": ["exact"],
        "qa_reviewer": ["exact"],
        "created": ["exact", "lt", "lte", "gt", "gte"],
        "modified": ["exact", "lt", "lte", "gt", "gte"],
    }
    ordering = ("-modified",)
    ordering_fields = ("title", "status", "type", "created", "modified")

    def get_queryset(self):
        return Article.objects.filter(
            Q(reviewer=self.profile, status=Article.Status.FEEDBACK)
            | Q(peer_reviewer=self.profile, status=Article.Status.PEER_REVIEW)
            | Q(qa_reviewer=self.profile, status=Article.Status.QA_REVIEW)
            | Q(author=self.profile, status=Article.Status.AUTHOR_APPROVAL)
        )

    def get_serializer_class(self):
        # TODO: Create a unique endpoint for creating instances so that we
        #       can assign a "new instance" to have better control over what
        #       serializer we want to use.
        # if self.kwargs.get("pk"):
        #     return ArticleFullModelSerializer
        # return ArticleModelSerializer
        return ArticleFullModelSerializer



from django.db import models
from slugify import slugify
from wbcore.models import WBModel


class ArticleType(WBModel):
    """An arbitrary type for articles.

    The type of an article helps filtering out undesired categories of
    articles.
    Furthermore, having the article type in a separate model allows adding,
    removing, and changing types during runtime.

    Attributes
    ----------
    label : models.CharField
        The unique label for this type.
        This is meant to be displayed to users.
    slug : models.SlugField
        A slugified version fo the label.
        This is used for searching ArticleTypes.
    parsers : models.ManyToManyField
        One or more optional parsers. The parser are used to convert the
        `content` of an article and its associated in-editor template(s)
        to something that can be published.
    """

    label = models.CharField(max_length=255, unique=True)
    slug = models.SlugField(max_length=255, null=True, blank=True, unique=True)
    parsers = models.ManyToManyField("wbwriter.PublicationParser", related_name="publication_parsers", blank=True)
    peer_reviewers = models.ManyToManyField("directory.Person", related_name="article_type_peer_reviewers")
    qa_reviewers = models.ManyToManyField("directory.Person", related_name="article_type_qa_reviewers")

    @classmethod
    def get_endpoint_basename(self):
        return "wbwriter:articletyperepresentation"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbwriter:articletyperepresentation-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{label}}"

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        self.slug = slugify(self.label)
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "Article Type"
        verbose_name_plural = "Article Types"



import factory

from wbwriter.factories.article import ArticleTypeFactory
from wbwriter.models import MetaInformation, MetaInformationInstance


class MetaInformationFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MetaInformation

    name = factory.Sequence(lambda n: f"metainfo {n}")
    key = factory.Sequence(lambda n: f"key {n}")

    @factory.post_generation
    def article_type(self, create, extracted, **kwargs):
        if not create:
            return

        self.article_type.add(ArticleTypeFactory())
        if extracted:
            for _type in extracted:
                self.article_type.add(_type)


class MetaInformationInstanceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MetaInformationInstance

    article = factory.SubFactory("wbwriter.factories.article.ArticleFactory")
    meta_information = factory.SubFactory(MetaInformationFactory)



import random

import factory
from django.contrib.contenttypes.models import ContentType
from wbcore.contrib.authentication.factories.users import InternalUserFactory
from wbcore.contrib.directory.factories.entries import PersonFactory

from wbwriter.models import (
    Article,
    ArticleType,
    Block,
    BlockParameter,
    DependantArticle,
    InEditorTemplate,
    Publication,
    PublicationParser,
    Template,
)
from wbwriter.publication_parser import PublicationParser as PublicationParserClass


class Parser(PublicationParserClass):
    def is_valid(self):
        return True


def content() -> dict:
    return {
        "sectionOrder": ["section-1"],  # A list of section keys that will be used in the `sections` dict.
        "sections": {
            "section-1": {
                "id": "section-1",
                "templateID": None,  # `null` means no InEditorTemplate instance, otherwise the ID of the instance.
                "configuration": None,  # `null` means no config. This is otherwise a dict: dict[str, Any]
                "content": {  # This is a dict: dict[str, str]. The actual content depends on the chosen template and user input.
                    "content": "lorem ipsum dolor...",
                },
            }
        },
    }


class ArticleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Article

    name = factory.Sequence(lambda n: f"article {n}")
    title = factory.Sequence(lambda n: f"title {n}")
    content = content()
    type = factory.SubFactory("wbwriter.factories.article.ArticleTypeFactory")
    author = factory.LazyAttribute(lambda o: InternalUserFactory.create().profile)
    feedback_contact = factory.LazyAttribute(lambda o: InternalUserFactory.create().profile)
    reviewer = factory.SubFactory("wbcore.contrib.directory.factories.entries.PersonFactory")
    peer_reviewer = factory.LazyAttribute(lambda o: InternalUserFactory.create().profile)
    peer_reviewer_approved = True
    qa_reviewer = factory.LazyAttribute(lambda o: InternalUserFactory.create().profile)
    template = factory.SubFactory("wbwriter.factories.article.TemplateFactory")
    status = Article.Status.DRAFT

    @factory.post_generation
    def tags(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for tag in extracted:
                self.tags.add(tag)


class DependantArticleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DependantArticle

    article = factory.SubFactory(ArticleFactory)
    dependant_article = factory.SubFactory(ArticleFactory)


class ArticleTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ArticleType

    label = factory.Sequence(lambda n: f"articletype {n}")

    @factory.post_generation
    def parsers(self, create, extracted, **kwargs):
        self.parsers.add(PublicationParserFactory.create())

    @factory.post_generation
    def peer_reviewers(self, create, extracted, **kwargs):
        if not create:
            return

        self.peer_reviewers.add(PersonFactory())
        if extracted:
            for peer in extracted:
                self.peer_reviewers.add(peer)

    @factory.post_generation
    def qa_reviewers(self, create, extracted, **kwargs):
        if not create:
            return

        self.qa_reviewers.add(PersonFactory())
        if extracted:
            for peer in extracted:
                self.qa_reviewers.add(peer)


class TemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Template

    title = factory.Sequence(lambda n: f"template {n}")
    template = factory.Faker("paragraph")

    @factory.post_generation
    def styles(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for style in extracted:
                self.styles.add(style)


class BlockFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Block

    title = factory.Sequence(lambda n: f"block {n}")
    html = factory.Faker("paragraph")


class BlockParameterFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BlockParameter

    block = factory.SubFactory(BlockFactory)
    title = factory.Sequence(lambda n: f"blockparameter {n}")
    order = random.randint(1, 9999)


class InEditorTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = InEditorTemplate

    uuid = factory.Sequence(lambda n: f"uid {n}")
    title = factory.Sequence(lambda n: f"title {n}")
    description = factory.Faker("paragraph")
    style = factory.Faker("paragraph")
    template = factory.Faker("paragraph")


class AbstractPublicationFactory(factory.django.DjangoModelFactory):
    object_id = factory.SelfAttribute("content_object.id")
    content_type = factory.LazyAttribute(lambda o: ContentType.objects.get_for_model(o.content_object))

    class Meta:
        exclude = ["content_object"]
        abstract = True


class PublicationParserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = PublicationParser
        django_get_or_create = ("parser_path",)

    title = factory.Sequence(lambda n: f"publicationparser {n}")
    parser_path = "wbwriter.factories.article"


class PublicationFactory(AbstractPublicationFactory):
    class Meta:
        model = Publication

    title = factory.Sequence(lambda n: f"publication {n}")
    # author = factory.SubFactory("wbcore.contrib.directory.factories.PersonFactory")
    content = factory.Faker("paragraph")
    parser = factory.SubFactory(PublicationParserFactory)
    content_object = factory.SubFactory(ArticleFactory)
    description = factory.Faker("paragraph")
    target = factory.Faker("pystr")



from wbcore.metadata.configs.titles import TitleViewConfig

from wbwriter.models import Publication


class PublicationTitleConfig(TitleViewConfig):
    def get_list_title(self):
        title = "Publications"
        if self.request.GET.get("content_type") and self.request.GET.get("object_id"):
            publications = Publication.objects.filter(
                content_type__id=self.request.GET.get("content_type"), object_id=self.request.GET.get("object_id")
            )
            if publications.exists():
                pub = publications.first()
                content_type_label = pub.content_type.model
                content_title = pub.content_object.get_publication_metadata()["title"]
                title = f"{title} of {content_type_label}: {content_title}"

        return title



import django_filters
from django.db.models import Q
from wbcore import filters

from wbwriter.models import Article, ArticleType


class ArticleFilter(filters.FilterSet):
    show_only_involved_articles = filters.BooleanFilter(
        initial=True,
        label="Show only my articles",
        method="get_my_articles",
    )
    type = filters.ModelMultipleChoiceFilter(
        label="Types",
        queryset=ArticleType.objects.all(),
        endpoint=ArticleType.get_representation_endpoint(),
        value_key=ArticleType.get_representation_value_key(),
        label_key=ArticleType.get_representation_label_key(),
    )

    def get_my_articles(self, queryset, name, value):
        if value:
            request = self.request
            user = request.user
            profile = user.profile
            return queryset.filter(
                Q(author=profile) | Q(reviewer=profile) | Q(peer_reviewer=profile) | Q(qa_reviewer=profile)
            )
        return queryset

    status = filters.MultipleChoiceFilter(
        label="Status", choices=Article.Status.choices, widget=django_filters.widgets.CSVWidget
    )

    class Meta:
        model = Article
        fields = {
            "title": ["icontains"],
            "tags": ["exact"],
            "author": ["exact"],
            "reviewer": ["exact"],
            "peer_reviewer": ["exact"],
            "qa_reviewer": ["exact"],
            "created": ["exact", "lt", "lte", "gt", "gte"],
            "modified": ["exact", "lt", "lte", "gt", "gte"],
            "is_private": ["exact"],
        }



from .article import (
    ArticleModelViewSet,
    ReviewerArticleModelViewSet,
    DependantArticleModelViewSet,
    ArticleRepresentionViewSet,
)
from .article_type import ArticleTypeModelViewSet, ArticleTypeRepresentationViewSet
from .in_editor_template import (
    InEditorTemplateModelViewSet,
    InEditorTemplateRepresentationViewSet,
)
from .meta_information import MetaInformationRepresentationViewSet
from .meta_information_instance import (
    MetaInformationInstanceModelViewSet,
    MetaInformationInstanceRepresentationViewSet,
)
from .publication import (
    PublicationModelViewSet,
    PublicationParserModelViewSet,
    PublicationParserRepresentationViewSet,
    PublicationRepresentationViewSet,
)



from wbcore.serializers import HyperlinkField, ModelSerializer, RepresentationSerializer

from wbwriter.models import InEditorTemplate


class InEditorTemplateModelSerializer(ModelSerializer):
    """Serializes the complete InEditorTemplate model."""

    class Meta:
        model = InEditorTemplate
        fields = (
            "id",
            "uuid",
            "title",
            "description",
            "style",
            "template",
            "modified",
            "is_stand_alone_template",
        )


class InEditorTemplateRepresentationSerializer(RepresentationSerializer):
    """Serializes the minimum number of fields of the InEditorTemplate model
    that are needed to identify a template."""

    _detail = HyperlinkField(reverse_name="wbwriter:in-editor-template-detail")

    class Meta:
        model = InEditorTemplate
        fields = (
            "id",
            "uuid",
            "title",
            "description",
            "is_stand_alone_template",
            "_detail",
        )



from .article import (
    ArticleFullModelSerializer,
    ArticleModelSerializer,
    _get_plugin_configs,
    can_administrate_article,
    DependantArticleModelSerializer,
    ArticleRepresentionSerializer,
)
from .article_type import (
    ArticleTypeModelSerializer,
    ArticleTypeRepresentationSerializer,
)
from .in_editor_template import (
    InEditorTemplateModelSerializer,
    InEditorTemplateRepresentationSerializer,
)
from .meta_information import (
    MetaInformationInstanceModelSerializer,
    MetaInformationInstanceRepresentationSerializer,
    MetaInformationModelSerializer,
    MetaInformationRepresentationSerializer,
)
from .publication import (
    PublicationModelSerializer,
    PublicationParserRepresentationSerializer,
    PublicationParserSerializer,
    PublicationRepresentationSerializer,
)



from django.dispatch import receiver
from dynamic_preferences.registries import global_preferences_registry
from wbcore.test.signals import custom_update_kwargs
from wbwriter.models import Article
from wbwriter.viewsets import ArticleModelViewSet, ReviewerArticleModelViewSet


@receiver(custom_update_kwargs, sender=ReviewerArticleModelViewSet)
def receive_kwargs_reviewer_article(sender, *args, **kwargs):
    if (obj := kwargs.get("obj_factory")) and (user := kwargs.get("user")):
        obj.reviewer = user.profile
        obj.status = Article.Status.FEEDBACK
        obj.author.employers.set({global_preferences_registry.manager()["directory__main_company"]})
        obj.save()
    return {}


@receiver(custom_update_kwargs, sender=ArticleModelViewSet)
def receive_kwargs_article(sender, *args, **kwargs):
    if obj := kwargs.get("obj_factory"):
        obj.author.employers.set({global_preferences_registry.manager()["directory__main_company"]})
        obj.save()
    return {}



from django.db import models
from django.template import Context
from django.template import Template as DjangoTemplate
from slugify import slugify


class Block(models.Model):
    title = models.CharField(max_length=512)
    key = models.CharField(max_length=512, null=True, blank=True)
    html = models.TextField(default="")

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.key = slugify(self.title)
        super().save(*args, **kwargs)

    def parse_parameters(self, parameters):
        for index, parameter in enumerate(self.parameters.all().order_by("order")):
            yield parameter.title, parameter.parse_parameter(parameters[index])

    def render(self, parameters):
        return DjangoTemplate(self.html).render(Context(dict(self.parse_parameters(parameters))))



import datetime
from importlib import import_module

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.functional import cached_property
from modeltrans.fields import TranslationField
from rest_framework.reverse import reverse
from slugify import slugify
from wbcore.contrib.tags.models import TagModelMixin


class Publication(TagModelMixin, models.Model):
    """A publication of anything.

    A publication stores content prepared for a specific platform or use
    case. For instance, it stores an HTML string for publishing something on
    web.
    """

    class Meta:
        verbose_name = "Publication"
        verbose_name_plural = "Publications"

    title = models.CharField(max_length=1024)

    slug = models.CharField(max_length=1024, blank=True)

    target = models.CharField(max_length=256)

    teaser_image = models.ImageField(blank=True, null=True, upload_to="writer/publication/teasers")
    thumbnail_image = models.ImageField(blank=True, null=True, upload_to="writer/publication/thumbnails")

    author = models.ForeignKey(
        "directory.Person",
        related_name="publication",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    created = models.DateField(
        verbose_name="Creation Date",
        auto_now_add=True,
        help_text="The date on which this has been created.",
    )

    modified = models.DateTimeField(
        verbose_name="Last Modification Datetime",
        auto_now=True,
        help_text="The date and time on which this has been modified last.",
    )

    description = models.TextField(default="", blank=True)

    content = models.TextField(default="")

    content_file = models.FileField(
        max_length=256,
        upload_to="writer/publication/content_files",
        blank=True,
        null=True,
    )

    i18n = TranslationField(fields=["title", "slug", "content"])

    parser = models.ForeignKey(
        "wbwriter.PublicationParser",
        related_name="parsed_publication",
        on_delete=models.PROTECT,
    )

    additional_information = models.JSONField(default=dict, null=True, blank=True)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey("content_type", "object_id")

    @classmethod
    def create_or_update_from_parser_and_object(cls, parser, generic_object):
        if hasattr(generic_object, "_build_dto") and callable(generic_object._build_dto):
            ctype = ContentType.objects.get_for_model(generic_object)
            pub, created = cls.objects.get_or_create(parser=parser, content_type=ctype, object_id=generic_object.id)
            publ_metadata = generic_object.get_publication_metadata()
            for k, v in publ_metadata.items():
                setattr(pub, k, v)
            pub.parser.parser_class(
                generic_object._build_dto(), datetime.date.today() if created else pub.created
            ).parse(pub)
            pub.save()

    @classmethod
    def get_endpoint_basename(self):
        return "wbwriter:publication"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbwriter:publicationrepresentation-list"

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super().save(*args, **kwargs)

    def get_tag_detail_endpoint(self):
        return reverse("wbwriter:publication-detail", [self.id])

    def get_tag_representation(self):
        return self.title


class PublicationParser(models.Model):
    """A parser meant to be used to parse something into a publication.

    Attributes
    ----------
    title : models.CharField
        The unique title for this parser.

    parser_path : models.CharField
        A dotted path to the parser file relatoive to the projects root
        (ROOT/projects/).

    created : models.DateField
        The date on which this publication has been created.

    content : models.TextField
        The content of this publication in text form.

    content_file : models.FileField
        An optional file attachment, that represents the publications content.
    """

    class Meta:
        verbose_name = "PublicationParser"
        verbose_name_plural = "PublicationParsers"

    title = models.CharField(
        max_length=1024,
        unique=True,
    )
    slug = models.CharField(max_length=1024, blank=True)
    parser_path = models.CharField(
        max_length=1024,
        unique=True,
    )

    @cached_property
    def parser_class(self):
        return import_module(self.parser_path).Parser

    @classmethod
    def get_endpoint_basename(cls):
        return "wbwriter:publicationparser"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbwriter:publicationparserrepresentation-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{title}}"

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super().save(*args, **kwargs)

    def __str__(self):
        return f"{self.title}"



from contextlib import suppress

from wbcore.contrib.directory.serializers import PersonRepresentationSerializer
from wbcore.contrib.tags.serializers import TagSerializerMixin
from wbcore.serializers import (
    CharField,
    HyperlinkField,
    ModelSerializer,
    RepresentationSerializer,
    TextAreaField,
    register_resource,
)

from wbwriter.models import Publication, PublicationParser


class PublicationParserRepresentationSerializer(RepresentationSerializer):
    """Serializes the ID and the title of the Publication Parser model."""

    _detail = HyperlinkField(reverse_name="wbwriter:publicationparser-detail")

    class Meta:
        model = PublicationParser
        fields = ("id", "title", "_detail")


class PublicationModelSerializer(TagSerializerMixin, ModelSerializer):
    """Serializes the all fields of the Publication model except for the slug field."""

    description = TextAreaField()
    title = CharField(read_only=True)
    _author = PersonRepresentationSerializer(source="author")
    author = CharField(read_only=True)
    _parser = PublicationParserRepresentationSerializer(source="parser")

    @register_resource()
    def pdf_file(self, instance, request, user):
        with suppress(ValueError):
            if instance.content_file:
                return {"pdf_file": instance.content_file.url}
        return {}

    class Meta:
        model = Publication
        fields = (
            "id",
            "title",
            "target",
            "author",
            "_author",
            "created",
            "modified",
            "content",
            "description",
            "teaser_image",
            "thumbnail_image",
            "tags",
            "_tags",
            "additional_information",
            "_additional_resources",
            "parser",
            "_parser",
            "content_type",
            "object_id",
        )
        ordering = ("-modified", "title", "target")


class PublicationRepresentationSerializer(RepresentationSerializer):
    """Serializes the ID and the title of the Publication model."""

    _detail = HyperlinkField(reverse_name="wbwriter:publication-detail")

    class Meta:
        model = Publication
        fields = ("id", "title", "target", "modified", "_detail")
        ordering = ("-modified", "title", "target")


class PublicationParserSerializer(ModelSerializer):
    class Meta:
        model = PublicationParser
        fields = ("id", "title", "slug", "parser_path")



from dataclasses import asdict, dataclass
from datetime import datetime
from io import BytesIO
from typing import Any


@dataclass(frozen=True)
class ArticleDTO:
    name: str
    slug: str
    title: str
    teaser_image: BytesIO | None
    created: datetime
    modified: datetime
    content: dict[str, str]
    is_private: bool
    article_structure: dict[str, str]
    status: str
    meta_information: list[dict[str, Any]]
    tags: list[int]

    def to_dict(self):
        return asdict(self)



from wbcore.serializers import ModelSerializer, RepresentationSerializer

from wbwriter.models import ArticleType


class ArticleTypeModelSerializer(ModelSerializer):
    class Meta:
        model = ArticleType
        fields = ("id", "label", "peer_reviewers", "qa_reviewers")


class ArticleTypeRepresentationSerializer(RepresentationSerializer):
    class Meta:
        model = ArticleType
        fields = ("id", "label")



from django import template
from django.core.exceptions import ValidationError
from django.shortcuts import get_object_or_404
from django.template import Context, Template
from slugify import slugify
from wbcore.markdown.template import load_registered_templatetags

from wbwriter.models import Article

register = template.Library()


@register.inclusion_tag("wbwriter/templatetags/table_of_contents.html", takes_context=True)
def table_of_contents(context, title="Table of contents", custom_enumeration=True):
    if "article" in context and context.get("article"):
        article = context.get("article")
        article_toc = article.get_article_structure()
        return {"title": title, "custom_enumeration": custom_enumeration, **article_toc}

    return {"not_working": "<i>Table of content is only rendered in the exported PDF Version.</i>"}


@register.simple_tag(takes_context=True)
def load_article(context, article_id, with_title=True, include_in_toc=True, enumeration_from_toc=True):
    prefix = ""
    level = 1

    article = get_object_or_404(Article, id=article_id)
    # Render the content
    template = ""
    for section in article.content["sectionOrder"]:
        template = article.content["sections"][section]["content"]["content"]

    loaded_template_tags = load_registered_templatetags()
    if with_title:
        if main_article := context.get("article"):
            if structure := main_article.article_structure.get(str(article_id)):
                prefix = structure.get("enumerator", "")
                level = structure.get("level", 1)

        anchor = "<a name='" + slugify(article.name) + "'></a>"
        template = f"<h1 class='header-{level}'>{prefix} {article.title}{anchor}</h1>\n" + template
    return Template(loaded_template_tags + template).render(Context(context))


@register.inclusion_tag("wbwriter/templatetags/cite_article.html", takes_context=True)
def cite_article(context, article_id, text="(see {e} {t})"):
    if "article" in context and context.get("article"):
        main_article = context.get("article")
        prefix = ""
        if structure := main_article.article_structure.get(str(article_id)):
            prefix = structure.get("enumerator", "")
            structure.get("level", 1)
        article = get_object_or_404(Article, id=article_id)
        return {"anchor": slugify(article.name), "text": text.format(e=prefix, t=article.title)}

    return {"not_working": "(article citations only work in a rendered pdf.)"}


def block_templatetag(block):
    def templatetag(context, *args):
        parameters = block.parameters.all().count()
        if parameters != len(args):
            raise ValidationError({"non_field_errors": "Not all fields were send to the server."})
        return block.render(args)

    return templatetag


# TODO do we need the following three lines of code? (13.07.2022 / CL)
# with suppress(ProgrammingError):
#     for block in Block.objects.all():
#         register.simple_tag(takes_context=True, name=block.key)(block_templatetag(block))



from django.db import models
from wbcore.models import WBModel


class InEditorTemplate(WBModel):
    """A template that the user can choose and configure in the context of
    the WYSIWYG editor.

    In-Editor templates define CSS styles that need to be applied in
    combination with the HTML template, and a number of other
    In-Editor templates that may be nested within the given In-Editor
    template.

    Note that templates can not allow themself to be nested inside them.

    Attributes
    ----------
    uuid : models.CharField
        A unique identifier. The `uuid` must pass as a valid CSS class and
        must less than 256 characters long.

        The regular expression for valid CSS classes: "-?[_a-zA-Z]+[_a-zA-Z0-9-]*"

        See section 4.1.3, second paragraph on
        https://www.w3.org/TR/CSS21/syndata.html#characters
    title : models.CharField
        A human readable title.
    description : models.TextField
        A descriptive text that outlines the use case and possible
        configurations of the template.
    style : models.TextField, optional
        The CSS that is required to properly display the HTML template.
    template : models.TextField
        The HTML that makes up the template.

        TODO: WRITE ABOUT SCHEMA AND CONFIGURATION OPTIONS.
    is_stand_alone_template : models.BooleanField, default=True
        Signifies whether this template can be used without a surrounding
        template (i.e. as root node of the article).
    """

    uuid = models.CharField(max_length=255, unique=True)
    title = models.CharField(
        verbose_name="Title",
        max_length=255,
        help_text="The title should be unique but doesn't need to be.",
    )
    description = models.TextField(
        verbose_name="Description",
        default="",
        help_text="A brief text that describes the use case for this template.",
    )
    style = models.TextField(
        verbose_name="Template CSS",
        default="",
        blank=True,
        null=True,
        help_text="The CSS that styles the templates HTML.",
    )
    template = models.TextField(
        verbose_name="Template HTML",
        default="",
        help_text="The HTML code of the template.",
    )
    # configuration_field_definitions = models.JSONField(
    #     verbose_name="Configuration Field Definitions",
    #     help_text="Maps configuration IDs to field definitions for rendering a form.",
    #     blank=True,
    #     null=True,
    # )
    # configuration_form_layout = models.JSONField(
    #     verbose_name="Configuration Form Layout",
    #     help_text="The field layout of the configuration form.",
    #     blank=True,
    #     null=True,
    # )
    modified = models.DateTimeField(
        verbose_name="Last modification date and time",
        auto_now=True,
        help_text="The last time this template has been edited.",
    )

    is_stand_alone_template = models.BooleanField(default=True)

    def __str__(self):
        return self.title

    @classmethod
    def get_endpoint_basename(self):
        return "wbwriter:in-editor-template"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbwriter:in-editor-template-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{ title }}"



from typing import Optional

from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
)
from wbcore.metadata.configs.display.instance_display.utils import repeat_field
from wbcore.metadata.configs.display.view_config import DisplayViewConfig


class PublicationDisplayConfig(DisplayViewConfig):
    """Provides getter methods for the list and instance displays."""

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                ["title", "author"],
                ["created", "modified"],
                [repeat_field(2, "teaser_image")],
                [repeat_field(2, "description")],
            ]
        )

    def get_list_display(self) -> Optional[dp.ListDisplay]:
        """Sets up and returns the list display for Publication."""
        return dp.ListDisplay(
            fields=[
                dp.Field(key="title", label="Title"),
                dp.Field(key="author", label="Author"),
                dp.Field(key="modified", label="Modified"),
                dp.Field(key="created", label="Created"),
                dp.Field(key="teaser_image", label="Teaser image"),
            ],
            formatting=[],
            legends=[],
        )


class PublicationParserDisplayConfig(DisplayViewConfig):
    """Provides getter methods for the list and instance displays."""

    def get_instance_display(self) -> Display:
        return create_simple_display([["title", "parser_path"]])

    def get_list_display(self) -> Optional[dp.ListDisplay]:
        """Sets up and returns the list display for Publication Parser."""
        return dp.ListDisplay(
            fields=[
                dp.Field(key="title", label="Title"),
                dp.Field(key="parser_path", label="Parser Path"),
            ],
            formatting=[],
            legends=[],
        )



# Generated by Django 4.2.9 on 2024-02-20 10:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbwriter", "0009_dependantarticle"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="article",
            options={
                "permissions": [("administrate_article", "Can administrate Articles.")],
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
            },
        ),
    ]



from django.apps import apps
from django.db import connection
from django.db.models.signals import pre_migrate
from pytest_factoryboy import register
from wbcore.contrib.authentication.factories import UserFactory
from wbcore.contrib.directory.factories import PersonFactory
from wbcore.contrib.geography.tests.signals import app_pre_migration
from wbwriter.factories import (
    ArticleFactory,
    ArticleTypeFactory,
    BlockFactory,
    InEditorTemplateFactory,
    MetaInformationFactory,
    MetaInformationInstanceFactory,
    PublicationParserFactory,
    TemplateFactory,
)

register(UserFactory)
register(ArticleFactory)
register(ArticleTypeFactory)
register(BlockFactory)
register(InEditorTemplateFactory)
register(PersonFactory)
register(PublicationParserFactory)
register(TemplateFactory)
register(MetaInformationFactory)
register(MetaInformationInstanceFactory)

from .signals import *

pre_migrate.connect(app_pre_migration, sender=apps.get_app_config("wbwriter"))



import wbcore.serializers as wb_serializers
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse, reverse_lazy
from wbcore.contrib.directory.models import Person
from wbcore.contrib.directory.serializers import (
    InternalUserProfileRepresentationSerializer,
)
from wbcore.contrib.documents.models import Document
from wbcore.contrib.tags.serializers import TagSerializerMixin
from wbcore.serializers import (
    HyperlinkField,
    ModelSerializer,
    PrimaryKeyRelatedField,
    RepresentationSerializer,
    TemplatedJSONTextEditor,
    register_only_instance_resource,
    register_resource,
)

from wbwriter.models import (
    Article,
    ArticleType,
    DependantArticle,
    can_administrate_article,
)

from .article_type import ArticleTypeRepresentationSerializer


class ArticleRepresentionSerializer(RepresentationSerializer):
    class Meta:
        model = Article
        fields = (
            "id",
            "title",
        )


class DependantArticleModelSerializer(ModelSerializer):
    _article = ArticleRepresentionSerializer(source="article")
    article_url = HyperlinkField(reverse_name="wbwriter:article-detail", id_field_name="article_id")
    _dependant_article = ArticleRepresentionSerializer(source="dependant_article")
    dependant_article_url = HyperlinkField(
        reverse_name="wbwriter:article-detail", id_field_name="dependant_article_id"
    )

    class Meta:
        model = DependantArticle
        fields = (
            "id",
            "article",
            "_article",
            "article_url",
            "dependant_article",
            "_dependant_article",
            "dependant_article_url",
        )


from wbcore.contrib.i18n.serializers.mixins import ModelTranslateSerializerMixin


class ArticleModelSerializer(ModelTranslateSerializerMixin, TagSerializerMixin, ModelSerializer):
    """Serializes a subset of the fields of of the Article model to minimize
    workload on serializing lists of instances.
    """

    author = PrimaryKeyRelatedField(
        many=False, label="Author", queryset=lambda: Person.objects.filter_only_internal()
    )  # we lazy load queryset to know evaluate them at runtime
    _author = InternalUserProfileRepresentationSerializer(source="author")
    _qa_reviewer = InternalUserProfileRepresentationSerializer(source="qa_reviewer", many=False)

    _type = ArticleTypeRepresentationSerializer(source="type")

    @register_resource()
    def generate(self, instance, request, user):
        return {
            "generate_pdf": reverse("wbwriter:article-generate-pdf", args=[instance.id], request=request),
            "edit": reverse("wbwriter:article-edit", args=[instance.id], request=request),
        }

    @register_resource()
    def metainformationinstance(self, instance, request, user):
        return {
            "metainformationinstance": f"{reverse('wbwriter:metainformationinstancearticle-list', args=[instance.id], request=request)}",
        }

    @register_resource()
    def dependencies(self, instance, request, user):
        return {
            "dependantarticle-article": reverse(
                viewname="wbwriter:dependantarticle-article-list", args=[instance.id], request=request
            ),
            "usedarticle-article": reverse(
                viewname="wbwriter:usedarticle-article-list", args=[instance.id], request=request
            ),
        }

    @register_only_instance_resource()
    def preview(self, instance, request, user, **kwargs):
        if document := Document.get_for_object(instance).filter(system_key=instance.system_key).first():
            return {"preview": document.file.url}
        return dict()

    @register_only_instance_resource()
    def reroll_reviewers(self, instance, request, user, **kwargs):
        if can_administrate_article(instance, user):
            return {
                "reroll_peer": reverse("wbwriter:article-reroll-peer", args=[instance.id], request=request),
                "reroll_qa": reverse("wbwriter:article-reroll-qa", args=[instance.id], request=request),
                "reroll_peer_and_qa": reverse(
                    "wbwriter:article-reroll-peer-and-qa", args=[instance.id], request=request
                ),
                "assign_new_author": reverse(
                    "wbwriter:article-assign-new-author", args=[instance.id], request=request
                ),
            }
        else:
            return {}

    @register_only_instance_resource()
    def publications(self, instance, request, user, view=None, **kwargs):
        if view and instance.publications.exists():
            return {
                "publications": f"{reverse('wbwriter:publication-list', request=request)}?content_type={view.content_type.id}&object_id={instance.id}"
            }
        return {}

    def create(self, validated_data):
        if request := self.context.get("request"):
            validated_data["author"] = request.user.profile
        return super().create(validated_data)

    def validate(self, data):
        articles = Article.objects.filter(name=data.get("name"))
        if self.instance is not None:
            articles = articles.exclude(id=self.instance.id)
            if data.get("title") in ["", None] and self.instance.title is None:
                data["title"] = data.get("name")

        if articles.exists():
            raise ValidationError({"name": ["Name already exists."]})

        return data

    class Meta:
        model = Article
        fields = (
            "id",
            "name",
            "title",
            "slug",
            "type",
            "_type",
            "status",
            "teaser_image",
            "created",
            "modified",
            "author",
            "_author",
            "qa_reviewer",
            "_qa_reviewer",
            "tags",
            "_tags",
            "_additional_resources",
            "_i18n",
        )


def _get_plugin_configs(request):
    return {
        "plugins": " ".join(
            [
                "advlist autolink lists link image charmap print preview anchor",
                "searchreplace visualblocks fullscreen noneditable template",
                "insertdatetime table advtable help wordcount code",
                "pagebreak hr imagetools powerpaste lance flite",
            ]
        ),
        "toolbar": " ".join(
            [
                "undo redo | formatselect | bold italic underline strikethrough backcolor | ",
                "alignleft aligncenter alignright alignjustify | ",
                "bullist numlist outdent indent | pastetext removeformat | help | code fullscreen | lance | ",
                "flite-toggletracking flite-toggleshow | flite-acceptone flite-rejectone | flite-acceptall flite-rejectall",
            ]
        ),
        "menu": {
            "file": {
                "title": "File",
                "items": "newdocument restoredraft | preview | print",
            },
            "edit": {
                "title": "Edit",
                "items": "undo redo | cut copy paste | selectall | searchreplace",
            },
            "view": {
                "title": "View",
                "items": "visualaid visualchars visualblocks | spellchecker | preview fullscreen",
            },
            "insert": {
                "title": "Insert",
                "items": " | ".join(
                    [
                        "image link template codesample inserttable",
                        "charmap emoticons hr",
                        "pagebreak nonbreaking anchor toc",
                        "insertdatetime",
                    ]
                ),
            },
            "format": {
                "title": "Format",
                "items": "bold italic underline strikethrough superscript subscript codeformat | formats blockformats"
                + " fontformats fontsizes align lineheight | forecolor backcolor | removeformat",
            },
            "tools": {
                "title": "Tools",
                "items": "code | spellchecker spellcheckerlanguage | wordcount",
            },
            "table": {
                "title": "Table",
                "items": "inserttable | cell row column | tableprops deletetable",
            },
            # "tc": {"title": "Comments", "items": "addcomment showcomments deleteallconversations"},
            "help": {"title": "Help", "items": "help"},
        },
        "paste_as_text": True,
        "powerpaste_word_import": "clean",
        "powerpaste_googledocs_import": "clean",
        "powerpaste_html_import": "clean",
        "browser_spellcheck": True,
        "atp": {
            "templates": reverse_lazy("wbwriter:in-editor-template-list", request=request),
        },
        "content_style": "body{margin:0px 12px;}",
        "deprecation_warnings": False,
    }


class ArticleFullModelSerializer(ArticleModelSerializer):
    """Serializes the full set of the fields of of the Article model."""

    is_private_icon = wb_serializers.IconSelectField(read_only=True)

    author = wb_serializers.PrimaryKeyRelatedField(
        many=False,
        label="Author",
        read_only=lambda view: (not view.can_edit_article_author or view.instance.status == Article.Status.APPROVED)
        if view.instance
        else not view.new_mode,
        queryset=lambda: Person.objects.filter_only_internal(),
        default=wb_serializers.CurrentUserDefault("profile"),
    )
    name = wb_serializers.CharField(
        max_length=1024,
        label="Name",
        help_text="A unique name to reference this article.",
        read_only=lambda view: not view.can_edit_article_meta_data if view.instance else not view.new_mode,
    )
    title = wb_serializers.CharField(
        max_length=1024,
        label="Title",
        required=False,
        help_text="The title of the article that is going to be used when imported into other articles. "
        + "Defaults to the name of the article when not set.",
        read_only=lambda view: not view.can_edit_article_meta_data if view.instance else not view.new_mode,
    )

    teaser_image = wb_serializers.ImageField(
        label="Teaser image",
        required=False,
        read_only=lambda view: not view.can_edit_article_content if view.instance else not view.new_mode,
    )

    content = TemplatedJSONTextEditor(
        templates="wbwriter:in-editor-template-list",
        default_editor_config=_get_plugin_configs,
        default="",
        label="Content",
        read_only=lambda view: not view.can_edit_article_content if view.instance else not view.new_mode,
    )

    type = wb_serializers.PrimaryKeyRelatedField(
        label="Type",
        many=False,
        queryset=ArticleType.objects.all(),
        read_only=lambda view: not view.can_edit_article_type if view.instance else not view.new_mode,
    )
    _type = ArticleTypeRepresentationSerializer(source="type", many=False)
    feedback_contact = wb_serializers.PrimaryKeyRelatedField(
        label="Feedback Contact",
        many=False,
        required=False,
        default=wb_serializers.DefaultAttributeFromObject(source="author.id"),
        queryset=lambda: Person.objects.filter_only_internal(),  # TODO: Filter out the people from the qa reviewer person group
    )
    _feedback_contact = InternalUserProfileRepresentationSerializer(source="feedback_contact", many=False)
    reviewer = PrimaryKeyRelatedField(many=False, label="Reviewer", read_only=True)
    _reviewer = InternalUserProfileRepresentationSerializer(source="reviewer", many=False)

    peer_reviewer = PrimaryKeyRelatedField(
        many=False,
        label="Peer Reviewers",
        queryset=lambda: Person.objects.filter_only_internal(),
        read_only=lambda view: not view.can_administrate_article and not view.new_mode,
    )
    _peer_reviewer = InternalUserProfileRepresentationSerializer(source="peer_reviewer", many=False)

    qa_reviewer = PrimaryKeyRelatedField(
        many=False,
        label="QA Reviewer",
        queryset=lambda: Person.objects.filter_only_internal(),
        read_only=lambda view: not view.can_administrate_article and not view.new_mode,
    )
    _qa_reviewer = InternalUserProfileRepresentationSerializer(source="qa_reviewer", many=False)

    def validate(self, data):
        data = super().validate(data)
        if "type" in data:
            if (
                self.instance
                and data.get("type")
                and data.get("type").slug != "ddq"
                and (
                    ("author" in data and not data.get("author"))
                    or ("author" not in data and self.instance.author is None)
                )
            ):
                raise serializers.ValidationError(
                    {"type": 'This must be set to "DDQ" as long as no author has been assigned!'}
                )

        return data

    class Meta:
        model = Article
        fields = (
            "id",
            "is_private_icon",
            "name",
            "title",
            "slug",
            "type",
            "_type",
            "created",
            "modified",
            "teaser_image",
            "content",
            "status",
            "author",
            "_author",
            "feedback_contact",
            "_feedback_contact",
            "reviewer",
            "_reviewer",
            "peer_reviewer",
            "_peer_reviewer",
            "qa_reviewer",
            "_qa_reviewer",
            "is_private",
            "tags",
            "_tags",
            "_additional_resources",
            "_i18n",
        )
